@@ -1,6 +1,4 @@
 /******************************************************************************
- * $Id: pj_gridinfo.c 2548 2014-09-17 06:21:09Z warmerdam $
- *
  * Project:  PROJ.4
  * Purpose:  Functions for handling individual PJ_GRIDINFO's.  Includes
  *           loaders for all formats but CTABLE (in nad_init.c).
@@ -41,7 +39,7 @@
  * TODO - mloskot: re-implement porting friendly assert
  */
 # define assert(exp)	((void)0)
-#else/
+#else
 # include <assert.h>
 #endif /* _WIN32_WCE */
 
@@ -52,7 +50,7 @@
 /************************************************************************/
 
 static int  byte_order_test = 1;
-#define IS_LSB	(((unsigned char *) (&byte_order_test))[0] == 1)
+#define IS_LSB	(1 == ((unsigned char *) (&byte_order_test))[0])
 
 static void swap_words( unsigned char *data, int word_size, int word_count )
 
@@ -227,7 +225,7 @@ int pj_gridinfo_load( projCtx ctx, PJ_GRIDINFO *gi )
 
             if( pj_ctx_fread( ctx, row_buf,
                               sizeof(double), gi->ct->lim.lam * 2, fid )
-                != 2 * gi->ct->lim.lam )
+                != (size_t)( 2 * gi->ct->lim.lam ) )
             {
                 pj_dalloc( row_buf );
                 pj_dalloc( ct_tmp.cvs );
@@ -305,7 +303,7 @@ int pj_gridinfo_load( projCtx ctx, PJ_GRIDINFO *gi )
 
             if( pj_ctx_fread( ctx, row_buf, sizeof(float),
                               gi->ct->lim.lam*4, fid )
-                != 4 * gi->ct->lim.lam )
+                != (size_t)( 4 * gi->ct->lim.lam ) )
             {
                 pj_dalloc( row_buf );
                 pj_dalloc( ct_tmp.cvs );
@@ -370,7 +368,7 @@ int pj_gridinfo_load( projCtx ctx, PJ_GRIDINFO *gi )
         }
 
         if( pj_ctx_fread( ctx, ct_tmp.cvs, sizeof(float), words, fid )
-            != words )
+            != (size_t)words )
         {
             pj_dalloc( ct_tmp.cvs );
             pj_release_lock();
@@ -399,7 +397,7 @@ int pj_gridinfo_load( projCtx ctx, PJ_GRIDINFO *gi )
 /*      Seek a parent grid file by name from a grid list                */
 /************************************************************************/
 
-static PJ_GRIDINFO* pj_gridinfo_parent( PJ_GRIDINFO *gilist, 
+static PJ_GRIDINFO* pj_gridinfo_parent( PJ_GRIDINFO *gilist,
         const char *name, int length )
 {
     while( gilist )
@@ -860,9 +858,8 @@ PJ_GRIDINFO *pj_gridinfo_init( projCtx ctx, const char *gridname )
 /* -------------------------------------------------------------------- */
     if( pj_ctx_fread( ctx, header, sizeof(header), 1, fp ) != 1 )
     {
-        pj_ctx_fclose( ctx, fp );
-        pj_ctx_set_errno( ctx, -38 );
-        return gilist;
+        /* some files may be smaller that sizeof(header), eg 160, so */
+        ctx->last_errno = 0; /* don't treat as a persistent error */
     }
 
     pj_ctx_fseek( ctx, fp, SEEK_SET, 0 );
